/**
 * @public
 */
export declare type TreeData<T = any> = {
    text?: string;
    value?: T;
    icon?: string | false;
    state: TreeNodeState;
    children: TreeData<T>[];
    contextmenu?: string | Function;
    component?: string | Function;
};
/**
 * @public
 */
export declare type TreeNodeState = {
    opened: boolean;
    selected: boolean;
    disabled: boolean;
    loading: boolean;
    highlighted: boolean;
    openable: boolean;
    dropPosition: DropPosition;
    dropAllowed: boolean;
};
/**
 * @public
 */
export declare type EventData<T = any> = {
    data: TreeData<T>;
    path: number[];
};
/**
 * @public
 */
export declare type ContextMenuData<T = any> = {
    data: TreeData<T>;
    path: number[];
    root: TreeData<T>[];
    parent?: any;
};
/**
 * @public
 */
export declare class DoubleClick {
    private timeout;
    private clicked;
    private timer;
    constructor(timeout?: number);
    onclick(singleClick: () => void): void;
}
/**
 * @public
 */
export declare function getContainerClassName(noDots: boolean | undefined): string;
/**
 * @public
 */
export declare function getNodeClassName<T>(data: TreeData<T>, last: boolean): string;
/**
 * @public
 */
export declare function getAnchorClassName<T>(data: TreeData<T>, hovered: boolean, path: number[]): string;
/**
 * @public
 */
export declare function getCheckboxClassName<T>(data: TreeData<T>, path: number[]): string;
/**
 * @public
 */
export declare function getRootClassName(checkbox: boolean | undefined, size: string | undefined, theme?: string): string;
/**
 * @public
 */
export declare function getIconClassName(icon: string | false | undefined): string;
/**
 * @public
 */
export declare function getOclClassName(path: number[]): string;
/**
 * @public
 */
export declare function getMarkerClassName<T>(data: TreeData<T>): string;
/**
 * @public
 */
export declare const enum DropPosition {
    empty = 0,
    up = 1,
    inside = 2,
    down = 3,
}
/**
 * @public
 */
export declare type DropData<T = any> = {
    sourceData: TreeData<T>;
    sourcePath: number[];
    targetData: TreeData<T>;
    targetPath: number[];
};
/**
 * @public
 */
export declare function getNodeFromPath<T>(rootData: TreeData<T>[], path: number[]): TreeData<T> | null;
/**
 * @public
 */
export declare function ondrag<T>(pageY: number, dragTarget: HTMLElement | null, dropTarget: HTMLElement | null, data: TreeData<T>[], dropAllowed?: (dropData: DropData<T>) => boolean, next?: () => void): void;
/**
 * @public
 */
export declare function ondragleave<T>(target: HTMLElement, data: TreeData<T>[]): void;
/**
 * @public
 */
export declare function ondrop<T>(target: HTMLElement, dragTarget: HTMLElement | null, data: TreeData<T>[], next: (dropData: DropData<T>) => void): void;
/**
 * @public
 */
export declare function clearMarkerOfTree<T>(tree: TreeData<T>): void;
/**
 * @public
 */
export declare function getId(path: number[], preid?: string): string | undefined;
